library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;
use IEEE.numeric_std.all;

--
-- vgaChars Example Project
-- Doug Hodson, May 2003
-- www.RetroMicro.com
--
entity chipIO is
   
   port(
      pin_sysclk  : in std_logic;                      -- master clock from external clock source

      -- misc buttons and leds
      pin_pushbtn : in std_logic;                      -- push button on XSA board
      pin_led     : out std_logic_vector(6 downto 0);  -- 7-segment LED
      
      -- Flash RAM connections
      pin_flash_ce_n  : out std_logic;  -- chip-enable
      
      -- parallel port connections
      --parport_d: in std_logic_vector(7 downto 0);
      pin_parport_d : in std_logic_vector(4 downto 0);
      pin_parport_s : out std_logic_vector(6 downto 3);
      
      -- vga port connections
      pin_vga_red0     : out std_logic;
      pin_vga_red1     : out std_logic;
      pin_vga_green0   : out std_logic;
      pin_vga_green1   : out std_logic;
      pin_vga_blue0    : out std_logic;
      pin_vga_blue1    : out std_logic;
      pin_vga_hsync_n  : out std_logic;
      pin_vga_vsync_n  : out std_logic;

      -- SDRAM pin connections
      pin_sdram_clkfb : in std_logic;                   -- feedback clock with PCB delays
      pin_sdram_clk   : out std_logic;                  -- clock to RAM
      pin_sdram_cke   : out std_logic;                  -- clock-enable
      pin_sdram_cs_n  : out std_logic;                  -- chip-select
      pin_sdram_ras_n : out std_logic;                  -- RAS
      pin_sdram_cas_n : out std_logic;                  -- CAS
      pin_sdram_we_n  : out std_logic;                  -- write-enable
      pin_sdram_ba    : out unsigned( 1 downto 0);      -- bank-address
      pin_sdram_addr  : out unsigned(11 downto 0);      -- address bus
      pin_sdram_data  : inout unsigned(16-1 downto 0);  -- data bus
      pin_sdram_dqmh  : out std_logic;                  -- DQMH
      pin_sdram_dqml  : out std_logic;                  -- DQML
      
      -- RS232 port connections
      pin_rs232_rd    : in std_logic;                   -- receive data
      pin_rs232_td    : out std_logic;                  -- transmit data
      pin_rs232_cts   : in std_logic;                   -- clear to send
      pin_rs232_rts   : out std_logic;                  -- request to send

      -- XStend-2 SRAM connections
      pin_sram_ce_n   : out std_logic;                  -- chip enable

      -- PS-2 interface
      pin_ps2_data    : in std_logic;
      pin_ps2_clk     : in std_logic
   );
   
end chipIO;

architecture arch of chipIO is

	constant YES:	std_logic := '1';
	constant NO:	std_logic := '0';
	constant HI:	std_logic := '1';
	constant LO:	std_logic := '0';

   signal sysClk   : std_logic;  -- system clock
   signal sysReset : std_logic;  -- system reset

	component VGA_SYNC
		port(
		clock_25Mhz : in std_logic;
		red, green, blue : in std_logic;
		red_out, green_out, blue_out : out std_logic;
		horiz_sync_out, vert_sync_out : out std_logic;
		pixel_row, pixel_column : out std_logic_vector(9 downto 0));
	end component;
	
	signal vga_pixel_col : std_logic_vector(9 downto 0);
	signal vga_pixel_row : std_logic_vector(9 downto 0);

	component char_rom
		port(
		addr : in std_logic_vector(8 downto 0);
		clk  : in std_logic;
		dout : out std_logic_vector(7 downto 0));
	end component;
	
	signal rom_addr : std_logic_vector(8 downto 0);
	signal rom_clk  : std_logic;
	signal rom_dout : std_logic_vector(7 downto 0);
   signal mux_out  : std_logic;
begin

   -- disable flash and sram
   pin_flash_ce_n <= '1';
   pin_sram_ce_n <= '1';

   -- define system clock and reset
   sysClk   <= pin_sysclk;
   sysReset <= not pin_pushbtn;

	pin_vga_red0   <= '0';
	pin_vga_green0 <= '0';
	pin_vga_blue0  <= '0';


   --
   -- CREATE COMPONENTS
   --

	vgadriver:
		VGA_SYNC
		port map(
		clock_25Mhz    => sysClk,
		red            => '1',
		green          => mux_out,
		blue           => mux_out,
		red_out        => pin_vga_red1,
		green_out      => pin_vga_green1,
		blue_out       => pin_vga_blue1,
		horiz_sync_out => pin_vga_hsync_n,
		vert_sync_out  => pin_vga_vsync_n,
		pixel_row      => vga_pixel_row,
		pixel_column   => vga_pixel_col
		);

	vgacharrom:
		char_rom
		port map(
		addr   => rom_addr,
		clk    => sysClk,
		dout   => rom_dout
	);

   rom_addr <= vga_pixel_row(9 downto 1);
   mux_out  <= rom_dout( (CONV_INTEGER(NOT vga_pixel_col(3 downto 1))) );
   
end arch;